function  [ARL,fdbk_bl,sched_bl] = CRFB2_avg_resp_level(arg1,stim,resp,fdbk,sched)

% CRFB2_avg_resp_level -- Average Response Levels (ARLs) for CRFeedback2.
% 
% ARL = CRFB2_avg_resp_level(ARL_params,stim_categ,resp_categ)
% [ARL fdbk_bl sched_bl] = ...
%    CRFB2_avg_resp_level(ARL_params,stim_categ,resp_categ,fdbk,sched)
%
% ARL_params = CRFB2_avg_resp_level(P)
% [ARL_params,P] = CRFB2_avg_resp_level 
%   When called with a single argument, that argument P must have been
%   prepared by CRFB2_params. When called with no args, P=CRFB2_params(0).
%   CRFB2_avg_resp_level creates a 2-by-1 ARL_params structure from scratch.
%   ARL_params(1) has window_width=28. ARL_params(2) has window_width=42.
%   Categorical (1-7) units of ARL_params.midrange_stim; not speeds, etc.
%
% ARL_params is then fed back to a second call to CRFB2_avg_resp_level,
% along with other arguments, to calculate the average response levels.
%   stim_categ is a N_trials by N_runs matrix of stimulus categories --
%   integers between 1 and P.design_params.N_categ.
%   resp_categ is a same-shaped matrix of observer's responses on the same
%   scale. There can be no missing data -- all sequences must be complete.
%   Optionally, more matrices can be passed for sanity checking.
%
% CRFB2_avg_resp_level partitions each stim-resp sequence in blocks defined
% by ARL_params.endpoints. The very first block is exceptionally short and
% is replicated ARL.demo_regularizer times.
%
% The result is a N_blocks by N_runs matrix of average response levels.
% fdbk_bl is a same-shaped matrix of the (constant) feedback for each block.
%
% CMUExper/fdbk1/data/fdbk1_ARL2.m is the analogous function for the Fdbk1
% experiment.  CMUExper/CatRat1/data/trend/avg_resp_level.m for CatRat1.
% See also CMUExper/CatRat1/data/trend/CR_slide_regress.m and regcoefs.m.
%
% See also CRFB2_data_by_sbj, fdbk1_ARL2, avg_resp_level, regcoefs.

% (c) Laboratory for Cognitive Modeling and Computational Cognitive
% Neuroscience at the Ohio State University, http://cogmod.osu.edu
%
% 1.0.0 2009-12-04 AP: Wrote it, based on CMUExper/fdbk1/data/fdbk1_ARL2.m



%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%   When called with no args, generate standard_job from scratch   %%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if (nargin==0)
    arg1 = CRFB2_params(0) ;  % The params that administered the experiment
end

if ((nargin==0) || (nargin==1))  % Generate ARL_params based on P
    P = arg1 ;
    N_categ = P.design_params.N_categ ;    % 7 response categories
    N_tasks = 2 ;                          % MOTION=1, TEXTURE=2
    UNIF = P.enum.UNIF ;   % 3=Uniform schedule. Used only for demo Block 1
    FAST_LOW = P.enum.FAST_LOW ;  % one of the two "real" schedules
    trials_block = P.design_params.trials_block ./ N_tasks ;   % [28 28 14]
    total_trials = sum(trials_block(P.design_params.distrib_sched(:,1))) ;

    %-- Translate params
    midrange_stim = (N_categ+1)/2 ;
    assert(abs(midrange_stim-round(midrange_stim))<.0001) ;  % must be integer
    demo_width = trials_block(UNIF) ;               % 14
    short_window_width = trials_block(FAST_LOW) ;   % 28 --> 3 per segment
    default_window_width = 1.5*short_window_width ; % 42 --> 2 per segment
    assert(abs(default_window_width-round(default_window_width))<.0001) ;

    %-- Make ARL params tailored for 28-trial blocks
    ARL_params.midrange_stim = midrange_stim ;
    ARL_params.cutoff = 2.0 ;  % for outlier correction inside regcoefs.m
    ARL_params.window_width = short_window_width ;
    ARL_params.endpoints = (demo_width:short_window_width:total_trials) ;
    ARL_params.demo_regularizer = short_window_width/demo_width ;
    assert(ARL_params.endpoints(end)==total_trials) ;
    
    %-- Make ARL params tailored for 42-trial blocks
    ARL_params(2) = ARL_params(1) ;  % copy
    ARL_params(2).window_width = default_window_width ;
    ARL_params(2).endpoints = (demo_width:default_window_width:total_trials) ;
    ARL_params(2).demo_regularizer = default_window_width/demo_width ;
    assert(ARL_params(2).endpoints(end)==total_trials) ;
    
    %-- Export ARL_params and P and return
    ARL = ARL_params ;
    fdbk_bl = P ;
    return
elseif ((nargin~=3) && (nargin~=4) && (nargin~=5))
    error('CRFB2:analysis:badInputs',...
          'CRFB2_avg_resp_level must have 0, 1, 3, 4 or 5 input arguments.') ;
%else ((nargin==3) || (nargin==4)) -- continue
end


%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%      When called with arguments, get down to business      %%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%-- Handle optional fdbk argument
if (nargin<4) ; fdbk = repmat(-1,size(stim)) ; end
if (nargin<5) ; sched = repmat(-1,size(stim)) ; end

%-- Decipher ARL_params==arg1
win_width = arg1.window_width ;
endpoints = arg1.endpoints ;
mid_stim = arg1.midrange_stim ;
cutoff = arg1.cutoff ;
demo_regularizer = arg1.demo_regularizer ;
N_trials = endpoints(end) ;   % 350 = 700/N_tasks 

[k,N_runs] = size(stim) ;
if (k~=N_trials)
    warning('CRFB2:analysis:badInputs',...
            'Non-standard sequence length: %d instead of %d.',k,N_trials) ;
end

%-- Handle the exceptional first block by doubling or tripling its number of trials
endpoint1 = endpoints(1) ;
assert(endpoint1 == win_width/demo_regularizer) ;

stim = [repmat(stim(1:endpoint1,:),demo_regularizer-1,1) ; stim] ;
resp = [repmat(resp(1:endpoint1,:),demo_regularizer-1,1) ; resp] ;
fdbk = [repmat(fdbk(1:endpoint1,:),demo_regularizer-1,1) ; fdbk] ;
sched = [repmat(sched(1:endpoint1,:),demo_regularizer-1,1) ; sched] ;
endpoints = endpoint1*(demo_regularizer-1) + endpoints ;

%--- Main loop: Do the sliding regressions and calculate the ARLs
% See CMUExper/fdbk1/data/fdbk1_ARL2.m, which was based on
% CMUExper/CatRat1/data/trend/avg_resp_level.m, which in turn called
% CMUExper/CatRat1/data/trend/CR_slide_regress.m to do the main work.
N_blocks = length(endpoints) ;
ARL = NaN(N_blocks,N_runs) ;
fdbk_bl = NaN(N_blocks,N_runs) ;
sched_bl = NaN(N_blocks,N_runs) ;

for b = 1:N_blocks
  %- Isolate the relevant trials for this block
  if (b==1)
      idx = 1:endpoints(1) ;
  else
      idx = (endpoints(b-1)+1):endpoints(b) ;
  end
  
  %- Fit a regression line (twice) for each segment
  for k = 1:N_runs
    coefs = regcoefs(stim(idx,k),resp(idx,k),cutoff) ; % <-- main work
    ARL(b,k) = coefs(2) * mid_stim + coefs(1) ;    % slope * X + intercept
    
    % Sanity check: Make sure that feedback=const within segments
    f = min(fdbk(idx,k)) ;
    if (f~=max(fdbk(idx,k)))
        warning('CRFB2:analysis:badInputs',...
            'Inconsistent feedback in run %d, block %d.',k,b) ;
        fdbk_bl(b,k) = mean(fdbk(idx,k)) ;
    else
        fdbk_bl(b,k) = f ;
    end
    
    % Sanity check: Make sure that sched=const within segments
    f = min(sched(idx,k)) ;
    if (f~=max(sched(idx,k)))
        warning('CRFB2:analysis:badInputs',...
            'Inconsistent schedule in run %d, block %d.',k,b) ;
        sched_bl(b,k) = mean(sched(idx,k)) ;
    else
        sched_bl(b,k) = f ;
    end 
  end
end

%---  Return ARL and possibly fdbk_bl
%%%%% End of file CRFB2_avg_resp_level.m
