function  H1 = CRFB2_data_by_sbj1(data,ARL_params,P)

%CRFB2_hitcount1 -- The inner loop of CRFB2_hitcount; one-subject data.
%
% H1 = CRFB2_data_by_sbj1(data,ARL_params,P)
%
% DATA is a matrix prepared by CRFB2_import_data, without any demo or
% invalid trials (see D.data and D.skip_idx). It should contain data
% for a single subject only. DATA has one row per trial and columns:
%   1=block        : 1-13
%   2=blk_trial    : Trial number in current block
%   3=schedule     : 1=FAST_LOW, 2=SLOW_HIGH, 3=UNIF
%   4=task         : 1=MOTION, 2=TEXTURE
%   5=categ        : 1 through 7. This is also the correct answer.
%   6=orient       : degrees from vertical
%   7=feedbackp    : 1=feedback, 0=no_feedback
%   8=response     : integer from 1 to N_categ
%   9=correct      : 0=incorrect=(resp~=categ) or 1=correct=(resp==categ)
%  10=RT           : from stim onset until the (valid) keypress, msec
%  11=bonus        : cumulative bonus score at trial end
%  12=exit_code    : 0=EC_OK, 1=EC_BAD_KEY, 2=EC_FAST_KEY
%
% H1 is a structure with the following fields:
%  - MOTION_data  : a [350x9] matrix for the MOTION trials
%  - TEXTURE_data : a [350x9] matrix for the TEXTURE trials, with fields:
%    1=block      : 1-13
%    2=blk_trial  : Trial number in current block
%    3=schedule   : 1=FAST_LOW, 2=SLOW_HIGH, 3=UNIF
%    4=categ      : 1 through 7. This is also the correct answer.
%    5=orient     : degrees from vertical
%    6=feedbackp  : 1=feedback, 0=no_feedback
%    7=response   : integer from 1 to N_categ
%    8=correct    : 0=incorrect=(resp~=categ) or 1=correct=(resp==categ)
%    9=RT         : from stim onset until the (valid) keypress, msec
%  - schedule28   : [13x1] 1=FAST_LOW, 2=SLOW_HIGH, 3=UNIF
%  - feedback28   : [13x1] 1=feedback, 0=no_feedback
%  - MOTION_ARL28 : [13x1] ARLs calculatd in 28-trial blocks, MOTION task
%  - TEXTURE_ARL28: [13x1] ARLs calculatd in 28-trial blocks, TEXTURE task
%  - schedule42   : [9x1] 1=FAST_LOW, 2=SLOW_HIGH, 3=UNIF
%  - feedback42   : [9x1] 1=feedback, 0=no_feedback
%  - MOTION_ARL42 : [9x1] ARLs calculatd in 42-trial blocks, MOTION task
%  - TEXTURE_ARL42: [9x1] ARLs calculatd in 42-trial blocks, TEXTURE task
%
% The first ARL in each sequence is calculated only from the 14 UNIF trials
% of the corresponding task. It is best to ignore it in the anlyses.
%
% See also CRFB2_data_by_sbj, CRFB2_avg_resp_level, CRFeedback2.

% (c) Laboratory for Cognitive Modeling and Computational Cognitive
% Neuroscience at the Ohio State University, http://cogmod.osu.edu
%
% 1.0.0 2009-12-04 AP: Wrote it, based on CMUExper/fdbk1/data/data_by_sbj.m

%-- Enumerated types
MOTION = P.enum.MOTION ;
TEXTURE = P.enum.TEXTURE ;

%-- Split the data by task
MOTION_data  = data((data(:,4)==MOTION) ,[1:3 5:10]) ;
TEXTURE_data = data((data(:,4)==TEXTURE),[1:3 5:10]) ;

%-- Calculate the average response levels for the short aggregation window
stim_categ = [MOTION_data(:,4) TEXTURE_data(:,4)] ;
resp_categ = [MOTION_data(:,7) TEXTURE_data(:,7)] ;
fdbk = [MOTION_data(:,6) TEXTURE_data(:,6)] ;
sched = [MOTION_data(:,3) TEXTURE_data(:,3)] ;

[ARL fdbk_bl sched_bl] = CRFB2_avg_resp_level(ARL_params(1),...
    stim_categ,resp_categ,fdbk,sched) ;

MOTION_ARL28 = ARL(:,1) ;
TEXTURE_ARL28 = ARL(:,2) ;

assert(all(fdbk_bl(:,1)==fdbk_bl(:,2))) ;     % sanity check
feedback28 = fdbk_bl(:,1) ;
assert(all(sched_bl(:,1)==sched_bl(:,2))) ;   % sanity check
schedule28 = sched_bl(:,1) ;

%-- Calculate the average response levels for the long aggregation window
[ARL fdbk_bl sched_bl] = CRFB2_avg_resp_level(ARL_params(2),...
    stim_categ,resp_categ,fdbk,sched) ;

MOTION_ARL42 = ARL(:,1) ;
TEXTURE_ARL42 = ARL(:,2) ;

assert(all(fdbk_bl(:,1)==fdbk_bl(:,2))) ;     % sanity check
feedback42 = fdbk_bl(:,1) ;
assert(all(sched_bl(:,1)==sched_bl(:,2))) ;   % sanity check
schedule42 = sched_bl(:,1) ;

%-- Pack and return
H1.MOTION_data = MOTION_data ;
H1.TEXTURE_data = TEXTURE_data ;
H1.schedule28 = schedule28 ;
H1.feedback28 = feedback28 ;
H1.MOTION_ARL28 = MOTION_ARL28 ;
H1.TEXTURE_ARL28 = TEXTURE_ARL28 ;
H1.schedule42 = schedule42 ;
H1.feedback42 = feedback42 ;
H1.MOTION_ARL42 = MOTION_ARL42 ;
H1.TEXTURE_ARL42 = TEXTURE_ARL42 ;

%---  Return H1
%%%%% End of CRFB2_hitcount1.M
