function  D = CRFB2_import_data(arg1,spec,P)

%IMPORT_CRFB2_DATA --  Import the raw data of CRFeedback2 Experiment
%
% D = import_CRFB2_data(data)
% D = import_CRFB2_data(fname)
% D = import_CRFB2_data(fname,spec,P)
%
% DATA is a matrix produced by CRFeedback2.  Alternatively, the first argument
% can be a string FNAME with the name of a raw data file in ASCII format.
% Default FNAME = fullfile(CRFeedback2_pathstr,'data','raw_data.dat')
% The DATA matrix (whether supplied directly or loaded from ASCII file)
% should contain 23 columns as documented in CRFeedback2 and CRFB2_params.
% SPEC = [VERBOSE CLOCKP], default []=[1 false], where:
%  * VERBOSE=0 suppresses messages, 1(=default) to have them printed in
%    the console window, or pass a fileID of an open ASCII file.
%  * The instrumentation diagnostics (e.g. 'clock', 'spare_ITI') are not
%    processed when CLOCKP is false (the default).
% P defaults to CRFB2_params(0). Used for data indices and design checks.
%
% import_CRFB2_data reads the data file and returns a structure array 
% D with one element per subject and the following fields:
%  * sbj
%  * group
%  * data -- a matrix with one row per trial and the following columns:
%     1=block        : 1-13
%     2=blk_trial    : Trial number in current block
%     3=schedule     : 1=FAST_LOW, 2=SLOW_HIGH, 3=UNIF
%     4=task         : 1=MOTION, 2=TEXTURE
%     5=categ        : 1 through 7. This is also the correct answer.
%     6=orient       : degrees from vertical
%     7=feedbackp    : 1=feedback, 0=no_feedback
%     8=response     : integer from 1 to N_categ
%     9=correct      : 0=incorrect=(resp~=categ) or 1=correct=(resp==categ)
%    10=RT           : from stim onset until the (valid) keypress, msec
%    11=bonus        : cumulative bonus score at trial end
%    12=exit_code    : 0=EC_OK, 1=EC_BAD_KEY, 2=EC_FAST_KEY
%  * dry_run -- 0=HUMAN_DATA, 1+=xxxx_DRY_RUN. See P.enum.dry_run_enum
%  * miss_idx -- a column-vector of indices in STIM and DATA
%  * skip_idx == miss_idx
% -----  Instrumentation diagnostics are not imported when clockp is false
%  * diagnostics -- matrix with one row per trial and columns:
%     1=clock       : seconds since beginning of session to stimulus onset
%     2=spare_ITI   : Milliseconds available until Trial-Begin-Schedule
%     3=delta_msec  : lower bound on RT accuracy [msec], see KbCheck
%     4=frame_count : number of frames (=successful Flip's) in MOTION stimulus
%     5=stim_msec   : Milliseconds stimulus was visible on the screen
%
% Example 1 (Assuming all data are in a single file 'raw_data.dat'):
%   D = CRFB2_import_data ;
%   H = CRFB2_data_by_sbj(D) ;   % See CMUExper/fdbk1/data/data_by_sbj.m
%   % For average response levels, see CMUExper/fdbk1/data/fdbk1_ARL2.m
%
% Example 2 (Read data for subjects 1,2,3, and 20):
%   Ss = [1,2,3,20] ; N = length(Ss) ;
%   for k=1:N D(k)=CRFB2_import_data(sprintf('sbj%03d.dat',Ss(k)),0);end
%
% Example 3 (Invoke shell command to append the data files):
%   cd(fullfile(CRFeedback2_pathstr,'data')) ; pwd
%   !cat sbj*.dat > raw_data.dat   % Mac OS X, Linux
%   %!copy sbj???.dat raw_data.dat   % Windows
%   D=CRFB2_import_data(fullfile(CRFeedback2_pathstr,'data','raw_data.dat'));
%
% See also CRFeedback2, CRFB2_data_by_sbj, ML_write_data, CRFB2_file_herald.

% (c) Laboratory for Cognitive Modeling and Computational Cognitive
% Neuroscience at the Ohio State University, http://cogmod.osu.edu
%
% 1.0.0 2009-10-20 AP: Customized for CRFeedback2. spec=[verbose clockp]
% 0.9.0 2009-04-17 AP: Converted from MLRoving01/prog/*.m

if (nargin < 1)
    arg1 = fullfile(CRFeedback2_pathstr,'data','raw_data.dat') ;
end
if (nargin < 2) ; spec = [] ; end
if (isempty(spec)) ; spec = [1 false] ; end  % default [verbose clockp]
if (nargin < 3) ; P = CRFB2_params(0) ; end

fid = spec(1) ;
clockp = spec(2) ;

if (ischar(arg1))    %%%  FNAME supplied
   fprintf(fid,'\nLoading ASCII file %s.  Please wait...\n',arg1) ;
   raw_data = load(arg1,'-ascii') ;
else  % DATA supplied
   raw_data = arg1 ;
   clear arg1 ;
end
[rows cols] = size(raw_data) ;
fprintf(fid,'\n%d rows and %d columns loaded.\n',rows,cols) ;

%-- Do a (superficial) consistency check
if (find(isnan(raw_data)))
  warning ('CRFeedback2:inconsistentData',...
       'NaNs found in raw_data. Check the raw ASCII file.') ;
else
  fprintf(fid,'The RAW_DATA matrix contains no missing values.\n') ;
end;


%%%%%%%%%%%%%%%%%  Slice the big matrix into variables
fprintf(fid,'\nSlicing the big matrix into variables, columnwise.\n') ;

I = P.data.indices ;

%- Bookkeeping
sbj   = raw_data(:,I.sbj) ;
group = raw_data(:,I.group) ;
session = raw_data(:,I.session) ;
block = raw_data(:,I.blk) ;
schedule = raw_data(:,I.schedule) ;
%demop = raw_data(:,I.demop) ;
expT = raw_data(:,I.expT) ;
blk_tr = raw_data(:,I.blk_tr) ;
dry_run = raw_data(:,I.dry_run) ;    % 0='normal data collection', >=1='debug'

%- Stimulus specification
task = raw_data(:,I.task) ;
categ = raw_data(:,I.categ) ;
orient = raw_data(:,I.orient) ;
feedbackp = raw_data(:,I.feedbackp) ;

%- Behavioral data
resp = raw_data(:,I.resp) ;
corr = raw_data(:,I.corr) ;      % 0='incorrect', 1='correct'
RT = raw_data(:,I.RT) ;          % msec from last stimulus onset
bonus = raw_data(:,I.bonus) ;    % cumulative bonus score at trial end
exitcode = raw_data(:,I.exitcode) ; % 0=OK, >=1=must_repeat

%- Instrumentation diagnostics. Ignored when clockp==false
clock = raw_data(:,I.clock) ;
spare_ITI = raw_data(:,I.spare_ITI) ;
delta_msec = raw_data(:,I.delta_msec) ;
frame_count = raw_data(:,I.frame_count) ;
stim_msec = raw_data(:,I.stim_msec) ;

%-- Clear the big raw matrix
clear raw_data rows cols ;
fprintf(fid,'Cleared the big matrix RAW_DATA.\n') ;


%%%%%%%%%%%%%%%%%  Partition data by subject and pack into a structure

%-- Determine the unique subject numbers
u_sbj = unique(sbj) ;
N_sbj = length(u_sbj) ;

%-- Pack and analyze the data for each individual subject
for k = 1:N_sbj
   curr_sbj = u_sbj(k) ;
   idx = find(sbj==curr_sbj) ;
   fprintf(fid,'Processing %5d trials for participant %2d... ',...
           length(idx),curr_sbj) ;
   D(k) = do_one(P,clockp,curr_sbj,group(idx),session(idx) ...
      ,block(idx),schedule(idx),expT(idx),blk_tr(idx),dry_run(idx) ...
      ,task(idx),categ(idx),orient(idx),feedbackp(idx) ...
      ,resp(idx),corr(idx),RT(idx),bonus(idx),exitcode(idx) ...
      ,clock(idx),spare_ITI(idx),delta_msec(idx) ...
      ,frame_count(idx),stim_msec(idx)...
      ) ; %#ok<AGROW>
   fprintf(fid,'Done.\n') ;
end

%--- Return D
%%%% End of the main function


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function  D = do_one(P,clockp,sbj,group,session ...
      ,block, schedule, expT, blk_tr, dry_run ...
      ,task, categ, orient, feedbackp ...
      ,resp, corr, RT, bonus, exitcode ...
      ,clock, spare_ITI, delta_msec ...
      ,frame_count, stim_msec ...
      ) %#ok<INUSL>

%-- Experimental design parameters. See CRFB2_params.
Dparams = P.design_params ;
N_blocks = size(Dparams.feedback_sched,1) ;

%-- Enumerated type for exitcodes
EC_OK = P.enum.EC_OK ;
EC_BAD_KEY = P.enum.EC_BAD_KEY ;
%EC_FAST_KEY = P.enum.EC_FAST_KEY ;
%EC_INSTRUM = P.enum.EC_INSTRUM ;

%-- Check that GROUP and DRY_RUN are consistent throughout
if (min(group)~=max(group))
    warning('CRFeedback2:inconsistentData',...
        ' min(group)=%d while max(group)=%d.',min(group),max(group)) ;
end
group = group(1) ;

if (min(dry_run)~=max(dry_run))
    warning('CRFeedback2:inconsistentData',...
        ' min(dry_run)=%d while max(dry_run)=%d.',min(dry_run),max(dry_run));
end
%dry_run = dry_run(1) ;

%-- Session should equal 1
if (any(session~=1))
    warning('CRFeedback2:inconsistentData',...
        ' Session is not always 1 for sbj %d.',sbj) ;
end

%-- Identify the valid trials and the demo trials
valid_trial_p = ((exitcode==EC_OK) | (exitcode==EC_BAD_KEY)) ;
miss_idx = find(~valid_trial_p) ;
skip_idx = miss_idx ;

% demo_idx = find(demop~=0) ;
% skip_idx = union(miss_idx,demo_idx) ;
% ok = (1:length(block))' ; ok(skip_idx) = [] ;  % valid experimental trials


%-- Verify certain design constraints for each block
u_block = unique(block) ;
N_u_blocks = length(u_block) ;

if (N_u_blocks ~= N_blocks)
   warning('CRFeedback2:inconsistentData',...
       ' Nonstandard number (%d) of blocks for sbj %d.', N_u_blocks,sbj) ;
end

for k = 1:N_blocks
    bl1 = u_block(k) ;
    idx = (valid_trial_p & (block==bl1)) ;
    
    %- schedule should be constant within a given block
    [counts,vals] = xtab1(session(idx)) ;
    if (length(vals)~=1)
       warning('CRFeedback2:inconsistentData',...
           ' Inconsistent schedule (%s) in block %d, sbj %d.', ...
           mat2str(vals),bl1,sbj) ;
    end
    
    %- feedbackp should be constant within a given block
    [counts,vals] = xtab1(feedbackp(idx)) ;
    if (length(vals)~=1)
       warning('CRFeedback2:inconsistentData',...
           ' Inconsistent feedbackp (%s) in block %d, sbj %d.', ...
           mat2str(vals),bl1,sbj) ;
    end
    
    %- task should be counterbalanced within a given block
    counts = xtab1(task(idx)) ;
    if ((length(counts)~=2) || (counts(1)~=counts(2)))
       warning('CRFeedback2:inconsistentData',...
           ' Task is not counterbalanced (%s) in block %d, sbj %d.', ...
           mat2str(counts),bl1,sbj) ;
    end
end


%-- Pack everything into a big structure and return
%     1=block        : 1-9
%     2=blk_trial    : Trial number in current block
%     3=schedule     : 1=FAST_LOW, 2=SLOW_HIGH, 3=UNIF
%     4=task         : 1=MOTION, 2=TEXTURE
%     5=categ        : 1 through 7. This is also the correct answer.
%     6=orient       : degrees from vertical
%     7=feedbackp    : 1=feedback, 0=no_feedback
%     8=response     : integer from 1 to
%     9=correct      : 0=incorrect=(resp~=categ) or 1=correct=(resp==categ)
%    10=RT           : from stim onset until the (valid) keypress, msec
%    11=bonus        : cumulative bonus score at trial end
%    12=exit_code    : 0=EC_OK, 1=EC_BAD_KEY, 2=EC_FAST_KEY
data = [block,blk_tr,schedule,task,categ,orient,feedbackp ...
       ,resp,corr,RT,bonus,exitcode ] ;
    
D.sbj = sbj ;
D.group = group ;
D.data = data ;
D.dry_run = dry_run(1) ;
D.miss_idx = miss_idx ;
%D.demo_idx = demo_idx ;
D.skip_idx = skip_idx ;

%-- Instrumentation diagnostic information, optional
%     1=clock       : seconds since beginning of session to stimulus onset
%     2=spare_ITI   : Milliseconds available until Trial-Begin-Schedule
%     3=delta_msec  : lower bound on RT accuracy [msec], see KbCheck
%     4=frame_count : number of frames (=successful Flip's) in MOTION stimulus
%     5=stim_msec   : Milliseconds stimulus was visible on the screen
if (clockp)
    diagnostics = [clock,spare_ITI,delta_msec,frame_count,stim_msec ] ;
    D.diagnostics = diagnostics ;
end

%--- Return D
%%%% End of subfunction DO_ONE

%%%%% End of import_CRFB2_data.M
