function  P = CRFB2_params(level)

%CRFB2_params -- Parameters for Motion Learning Roving Experiment 1.
%
% P = CRFB2_params()
% P = CRFB2_params(level)
%
% Master parameter structure for the main program CRFeedback2 and its
% subfunctions do_CRFB2_trial, CRFB2_prepare, etc.
%
% LEVEL 1 (default): as used in the main CRFeedback2 
% LEVEL 0: omit fields 'sounds' and 'LUT'
% LEVEL 9 (debug): include additional fields and plots
%
% See also CRFeedback2, do_CRFB2_trial, CRFB2_prepare, RDC_params.

% (c) Laboratory for Cognitive Modeling and Computational Cognitive
% Neuroscience at the Ohio State University, http://cogmod.osu.edu
%
% 1.1.1 2009-11-16 AP: New group quotas for iScope1, iScope2, iScope3
% 1.1.0 2009-10-22 AP: 13 blocks; delta_log2_freq=0.4
% 1.0.2 2009-10-21 AP: Deployed on iScope3
% 1.0.1 2009-10-20 AP: Enum types. Low-freq is category 7, high-freq is 1.
% 1.0.0 2009-05-28 AP: Introduced Filter Params, etc.
% 0.9.0 2009-04-17 AP: Converted from MLRoving01/prog/*.m

if (nargin < 1) ; level = 1 ;   end   % default

P.descr = 'CRFeedback2 [v.1.1.1, 2009-11-16]' ;


%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%   EQUIPMENT CHARACTERISTICS   %%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% See ~/a/w/work/MLExper/calibration/ for details.
% 21" NEC AccuSync 120-BK CRT Monitors in all three cubicles
% There may be slight anisotropies, as usual.

%-- Retrieve the name of this computer from $HOSTNAME environ variable
host = hostname ;
equip_params.host = host ;
%intended_hosts = {host} ;   % uncomment this to disable later warnings
intended_hosts = {'iscope1','iscope2','iscope3'} ;
equip_params.intended_hosts = intended_hosts ;

if (any(strcmpi(host,intended_hosts)))
   deployedp = true ;    % used for actual data collection
else
   deployedp = false ;   % software development mode, dual monitor, etc.
end
equip_params.deployedp = deployedp ;

%-- Make sure we're running PsychToolbox 3.x
if (isempty(which('PsychtoolboxVersion')))
	PsychToolbox_avail = false ;
elseif (isnumeric(PsychtoolboxVersion))   % hence ver 2.x
	PsychToolbox_avail = false ;
else
    [verString,verStruct] = PsychtoolboxVersion ;
    if (verStruct.major < 3)
        PsychToolbox_avail = false ;
    else
	    PsychToolbox_avail = true ;
    end
end
P.PsychToolbox_avail = PsychToolbox_avail ;

%-- Designate which monitor is to be used for stimulus presentation
if (deployedp || ~PsychToolbox_avail)
    whichScreen = 0 ;  % Present everything on a single (mirrored) monitor
else   % software development mode
    whichScreen = max(Screen('Screens')) ;  % Debug on dual monitor if avail
end
equip_params.whichScreen = whichScreen ;


%%%%%  GAMMA Estimates   *********************************************
% Via utils\calibration\NewCalibratorGray.m

if (strcmpi(host,'iScope1'))
    GAMMA = 1.84 ;    % 2009-03-18 G=1.842, NVH
    %GAMMA = 1.80 ;    % 2008-07-28 G=1.79; 2008-06-11 G=1.80
    % Mirrored display configuration. 1024x768 @ 96 Hz. Nick Van Horn.
    % Luminance readings: black=1.25 cd/m2, grey=15.9, white=33.7; 2009-03-18

elseif (strcmpi(host,'iScope2'))
    GAMMA = 1.90 ;    % 2009-02-12, NEC2, 75Hz, Taylor Hayes
    %Previous: G=1.93 2008-12-09; G=1.89 2008-07-21
    % Mirrored display configuration. 1024x768 @ 75 Hz. Taylor Hayes
    % Luminance measures: 2008-09-11 on NEC2 using gamma=1.91
    %  black=0.50 ftL = 1.85 c;  grey= 16.7 =61.8 ; white= 32.0 = 118.4

elseif (strcmpi(host,'iScope3'))
    GAMMA = 1.80 ;    % 2009-03-18 G=1.73, NVH
    % Estimated GAMMA = 1.93 for with "NEC" CRT on iScope3, 2009-04-14.
    % Mirrored display configuration. 1024x768 @ 96 Hz. Nick Van Horn.
    % Luminance readings: black=1.47 cd/m2, grey=15.7, white=32.5; 2009-04-14

else
    GAMMA = 1.80 ;    % default
end

mm_pix = 98/256 ;      % 2008-10-16 on iScope1; 2008-12-09 on iScope2
pix_deg = 42.25 ;      % pixels/deg. Set so viewing_dist=927mm
equip_params.pix_deg = pix_deg ;
mm_deg = mm_pix * pix_deg ;              % milimeters per degree
equip_params.viewing_dist = round(mm_deg*180/pi) ;  % 927mm
% The full screen at 1024x768 resolution thus spans approx 26.2 x 19.7 deg.

%if (true)  % for software development of time_params
if (deployedp)
    frames_sec = 96 ;         % on iScope1 or iScope2, for data collection
    % On iScope2, the average stim_msec to frame_count ratio is 10.436 ms,
    % which corresponds to 95.82 Hz. See CRFB2_import_data diagnostics.
elseif (strcmpi(host,'vega'))
    frames_sec = 60 ;         % on Vega, for software development only
elseif (strcmpi(host,'triton'))
    frames_sec = 85 ;         % on Triton, for software development only
else
    frames_sec = 96 ;
end
equip_params.frames_sec = frames_sec ;   % frames per second (=Hz)
msec_frame = 1000/frames_sec ;           % msec per frame
equip_params.msec_frame = msec_frame ;
P.equip_params = equip_params ;


%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%   EXPERIMENTAL DESIGN   %%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% The main dependent variable is the rating on a 7-point scale,
% though RTs are recorded too.  Two stimulus types, chosen to be as
% dissimilar as possible within the visual modality:
%  1 = Random-dot cinematograms (RDCs) in which the dots all move along
%      parallel tracks. The attribute being rated is the global speed of
%      motion. The direction of motion is randomized across trials.
%  2 = Stationary filtered-noise patches. The amplitude spectrum of the
%      filter is a (pair of) Gaussians -- the same as the spectrum of a
%      sinusoidal grating. The filter is applied to a white-noise patch.
%      The attribute being rated is the (modal) spatial wavelength, labeled
%      "grain size" in the instructions. The orientation is randomized.

task_enum = {'MOTION', 'TEXTURE'} ;
enum.task_enum = task_enum ;
for k = 1:length(task_enum)
    enum.(task_enum{k}) = k ;   % enum.MOTION = 1; etc.
end
MOTION = enum.MOTION ;
TEXTURE = enum.TEXTURE ;
%P.enum = enum ;

% There is one stimulus per trial. Each block is a mixture of both
% stimulus types.  The presentation frequencies within a block are
% complementary.  There are three kinds of blocks (or "contexts"):
%  * fast_low  -- 28 motion stimuli, most of which have fast speeds, plus
%                 28 textures, most of which have low spatial wavelengths.
%  * slow_high -- 28 motion stimuli, most of which have slow speeds, plus
%                 28 textures, most of which have high spatial wavelengths.
%  * uniform   -- 14=2*7 motion stimuli plus 14=2*7 textures
%
% Compare with ~/work/CMUExper/fdbk1/data/make_fdbk1_stimuli.m
% Also ~/work/models/anchor/simul/schedule.m  and generate_stimuli.m,
% which are modeled after the LISP code in CatRat/programs/schedule.lisp

block_enum = {'FAST_LOW', 'SLOW_HIGH', 'UNIF'} ;
enum.block_enum = block_enum ;
for k = 1:length(block_enum)
    enum.(block_enum{k}) = k ;   % enum.FAST_LOW = 1; etc.
end
FAST_LOW = enum.FAST_LOW ;
SLOW_HIGH = enum.SLOW_HIGH ;
UNIF = enum.UNIF ;
%P.enum = enum ;

N_categ = 7 ;    % Number of response categories
design_params.N_categ = N_categ ;

% Uniform, triangular-high, and triangular-low presentation frequencies
if (N_categ==7)
    %U_categ = [1 1 1 1  2 2 2 2  3 3 3 3  4 4 4 4  5 5 5 5  6 6 6 6  7 7 7 7]' ;
    Un_categ = [1 1  2 2  3 3  4 4  5 5  6 6  7 7]' ;
    Hi_categ = [1  2 2  3 3 3  4 4 4 4  5 5 5 5 5  6 6 6 6 6 6  7 7 7 7 7 7 7]' ;
    Lo_categ = 8 - flipud(Hi_categ) ;
    %Lo_categ = [1 1 1 1 1 1 1  2 2 2 2 2 2  3 3 3 3 3  4 4 4 4  5 5 5  6 6  7]' ;
else
    error('CRFB2_params is hardwired for 7 response categories.') ;
end

% A "stimulus descriptor" is a [1x2] vector specifying the type (MOTION or
% TEXTURE) and the category (1:7). A "block descriptor template" is a
% set [N_trials x 2] of stimulus descriptors.
UNIF_MOTION_block_templ(:,2) = Un_categ ;
UNIF_MOTION_block_templ(:,1) = MOTION ;
UNIF_TEXTURE_block_templ(:,2) = Un_categ ;
UNIF_TEXTURE_block_templ(:,1) = TEXTURE ;
block_templ{UNIF} = [UNIF_MOTION_block_templ ; UNIF_TEXTURE_block_templ] ;
% xtab2(block_templ{UNIF}) ;

FAST_block_templ(:,2) = Hi_categ ;   % preponderance of fast stimuli
FAST_block_templ(:,1) = MOTION ;
LOW_block_templ(:,2)  = Lo_categ ;   % preponderance of low-freq stimuli
LOW_block_templ(:,1)  = TEXTURE ;
block_templ{FAST_LOW} = [FAST_block_templ ; LOW_block_templ] ;
% xtab2(block_templ{FAST_LOW}) ;

SLOW_block_templ(:,2) = Lo_categ ;   % preponderance of slow stimuli
SLOW_block_templ(:,1) = MOTION ;
HIGH_block_templ(:,2) = Hi_categ ;   % preponderance of high-freq stimuli
HIGH_block_templ(:,1) = TEXTURE ;
block_templ{SLOW_HIGH}= [SLOW_block_templ ; HIGH_block_templ] ;
design_params.block_templ = block_templ ;
% xtab2(block_templ{SLOW_HIGH}) ;

% The number of trials per block is not constant:
trials_block(UNIF) = size(block_templ{UNIF},1) ;
trials_block(FAST_LOW) = size(block_templ{FAST_LOW},1) ;
trials_block(SLOW_HIGH) = size(block_templ{SLOW_HIGH},1) ;
design_params.trials_block = trials_block ;

% Visual feedback (a big white digit) is presented during feedback blocks.
% A non-informative (big white) X is presented during no-feedback blocks.
% Feedback and no-feedback blocks alternate within Ss and the schedules
% are counterbalanced between Ss.
fdbk_1st_sched = [1, 1 1 1, 0 0 0, 1 1 1, 0 0 0]' ;
fdbk_2nd_sched = [1, 0 0 0, 1 1 1, 0 0 0, 1 1 1]' ;
N_blocks = length(fdbk_1st_sched) ;
assert(length(fdbk_2nd_sched)==N_blocks) ;


% EXPERIMENTAL GROUPS ******************************************
%
% There are 4 experimental groups in a 2x2 between-sbj factorial design.
% The feedback schedule is one factor: feedback-first vs feedback-second.
% The other factor is the stimulus distribution: FAST_LOW vs SLOW_HIGH.
% With the exception of the very first block (which is always uniform and
% has half as many trials), the stimulus distrib is fixed for each group.

group_descr = {'FastLow1','FastLow2','SlowHigh1','SlowHigh2'} ;
design_params.group_descr = group_descr ;
N_groups = length(group_descr) ;
design_params.N_groups = N_groups ;

%--- Feedback schedule
% Groups FastLow1 and SlowHigh1 have feedback-first schedule.
% Groups FastLow2 and SlowHigh2 have feedback-second schedule.
% Usage: feedbackp = feedback_sched(block,group) ;
feedback_sched = zeros(N_blocks,N_groups) ;
feedback_sched(:,1) = fdbk_1st_sched ;    % FastLow1
feedback_sched(:,2) = fdbk_2nd_sched ;    % FastLow2
feedback_sched(:,3) = fdbk_1st_sched ;    % SlowHigh1
feedback_sched(:,4) = fdbk_2nd_sched ;    % SlowHigh2
design_params.feedback_sched = feedback_sched ;

%--- Stimulus-distribution schedule
% Groups FastLow1 and FastLow2 have FAST_LOW schedule.
% Groups SlowHigh1 and SlowHigh2 have SLOW_HIGH schedule.
% The first block is always uniform in all groups.
% Usage:  schedule = distrib_sched(block,group) ;          % CRFeedback2
%         bl_templ = design_params.block_templ{schedule} ; % CRFB2_do_block
distrib_sched = zeros(N_blocks,N_groups) ;
distrib_sched(1,:) = UNIF ;     % block 1 is uniform in all groups
distrib_sched(2:N_blocks,[1,2]) = FAST_LOW ;
distrib_sched(2:N_blocks,[3,4]) = SLOW_HIGH ;
design_params.distrib_sched = distrib_sched ;

%--- Subject numbers: non-overlapping ranges for each host computer
if (strcmpi(host,'iscope1'))
    design_params.sbj_range = [451 466] ;
    %subjects_per_group = 4 ;    % 16 Ss on iScope1
    %group_quotas = [(1:N_groups)' repmat(subjects_per_group,N_groups,1)] ;
    group_quotas = [1 4 ; 2 3 ; 3 4 ; 4 4] ;   % 4+3+4+4=15 Ss on iScope1
elseif (strcmpi(host,'iscope2'))
    design_params.sbj_range = [467 482] ;
    %subjects_per_group = 4 ;    % 16 Ss on iScope2
    %group_quotas = [(1:N_groups)' repmat(subjects_per_group,N_groups,1)] ;
    group_quotas = [1 4 ; 2 4 ; 3 4 ; 4 3] ;   % 4+4+4+3=15 Ss on iScope2
elseif (strcmpi(host,'iscope3'))
    design_params.sbj_range = [483 498] ;
    %subjects_per_group = 4 ;    % 16 Ss on iScope3
    %group_quotas = [(1:N_groups)' repmat(subjects_per_group,N_groups,1)] ;
    group_quotas = [1 2 ; 2 3 ; 3 2 ; 4 3] ;   % 2+3+2+3=10 Ss on iScope1
else
    design_params.sbj_range = [1 9999] ;   % For software development
    subjects_per_group = 12 ;   % 48 Ss total
    group_quotas = [(1:N_groups)' repmat(subjects_per_group,N_groups,1)] ;
end
design_params.group_quotas = group_quotas ;

design_params.max_session = 1 ;


%--- Dot-speed distributions for MOTION stimuli
speed_mean = (1:N_categ) + 5.0 ;   % degrees per second, linear spacing
speed_std = 1.0 * (speed_mean./speed_mean(1)) ;  % Eckman's law
design_params.speed_mean = speed_mean ;
design_params.speed_std = speed_std ;


%--- Filter frequencies for TEXTURE stimuli
% We flip the order of spatial frequencies because wavelength are inversely
% related to frequencies. Thus, response category 1 has low wavelength
% (=high freq) textures and category 7 has high wavelength (=low freq).
%log2_center_freq = fliplr((0:.5:(N_categ-1)/2)) ;  % prior to 2009-10-22
delta_log2_freq = 0.4 ;                  % octaves
log2_center_freq = (N_categ-1: -1: 0) .* delta_log2_freq ;
center_freq = 2.^log2_center_freq ;      % cyclese per degree
bandwidth = 0.5 ;                        % octaves
design_params.log2_center_freq = log2_center_freq ;
design_params.center_freq = center_freq ;
design_params.bandwidth = bandwidth ;


%--- Range for randomized texture orientations and motion directions
design_params.orient_range = [-179 180] ;    % degrees from vertical

%--- Feedback characters for each category
feedback_char = '123456789' ;
nofeedback_char = 'X' ;
design_params.feedback_char = feedback_char(1:N_categ) ;
design_params.nofeedback_char = repmat(nofeedback_char,1,N_categ) ;

%--- Demo parameters
% None, because the demo is hard-wired in CRFeedback2.m

P.design_params = design_params ;


%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%   RDC PARAMETERS   %%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
%
% The MOTION stimuli are random-dot cinematograms (RDCs) presented in
% the fovea. All dots move along parallel tracks (N_coher=N_dots).
% The attribute being rated is the global speed of motion. The direction
% of motion is randomized across trials. The distribution of dot speeds is
% Gaussian with mean = Mparams.speed_mean(speed_idx) and standard deviation
% proportional to the mean (in agreement with Eckman's law).
% See CRFB2_make_RDC_spec for the exact algorithm.
% The dots are rendered via Screen.DrawDots inside CRFB2_do_trial.

Mparams.speed_mean = design_params.speed_mean ;  % see CRFB2_make_RDC_spec
Mparams.speed_std = design_params.speed_std ;

Mparams.pix_deg = pix_deg ;
Mparams.msec_frame = msec_frame ;

% Ideally, MOTION stimuli have indefinite duration. That is, the motion
% continues until the observer presses a key. To be able to generate a
% RDC_spec in advance, however, we need a finite duration. Response-time
% data from 55 observers in Feedback1 experiment shows that 97% of all
% responses are given in less than 5 seconds (in a distance rating task).
max_duration_seconds = 8.000 ;   % loop back after that many seconds
Mparams.max_duration_seconds = max_duration_seconds ;
duration_frames = round(max_duration_seconds*frames_sec) ;
Mparams.duration_frames = duration_frames ;
% Note that the actual physical refresh rate of NEC1 under iScope1
% is ~98 Hz, although Screen('FrameRate',0) reports 96 Hz...

Mparams.aperture = 7 ;              % diameter of circular aperture, deg
Mparams.N_dots = 150 ;
% CRFB2_make_RDC_spec is hard-wired for 100% coherence
Mparams.dot_diam = 0.10 ;           % 6 arcmin dot diameter
% CRFB2_make_RDC_spec is hard-wired for wraparound==1
P.Mparams = Mparams ;

if (level==9)  % debug mode. See CRFB2_do_trial for the real thing
    fig_handle = 1 ;
    figure(fig_handle) ; clf ;
    spec = [4, 0] ;  % [speed_idx, theta]
    RDC_spec = CRFB2_make_RDC_spec(P,spec) ;
    RDC_movie(Mparams,RDC_spec) ;        % fake movie, see CRFB2_do_trial
end


%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%  LOOK-UP TABLE PARAMETERS  %%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

LUT_params.GAMMA = GAMMA ;
LUT_params.PixelSize = 32 ;  % 32-bit depth; see ScreenPixelSize
LUT_params.ClutSize = 256 ;  % See Screen LoadNormalizedGammaTable?
if (PsychToolbox_avail)
    black = BlackIndex(whichScreen) ;
    white = WhiteIndex(whichScreen) ;   % foreground color index
else
    black = 0 ;      % 0=black
    white = 255 ;    % 255=white
end
LUT_params.black = black ;
LUT_params.white = white ;
grey = ceil((black+white)/2) ;
LUT_params.grey = grey ;
LUT_params.span = abs(LUT_params.white - LUT_params.black) ;
LUT_params.multiplier = min(white-grey,grey-black) ;  %MAKE_MLS03_FILTNOISE
LUT_params.backgr = grey ;     % empty screen
LUT_params.foregr = black ;    % for messages, fixation, etc.
LUT_params.feedback = white ;  % for feedback digits

%-- Make a linear gamma table that inverts the GAMMA function of the monitor.
% CRFB2_PREPARE replicates it 3 times to construct a full 256x3 table.
if (white > black)       % white(=255) on black(=0) ?
    luminance = [0:LUT_params.span]'./LUT_params.span ; %#ok<NBRAK>
else  % should never happen but just in case...
    luminance = [LUT_params.span:-1:0]'./LUT_params.span ; %#ok<NBRAK>
end
if (level==9) ; LUT_params.lumin = luminance ; end   % debug mode

gamma_table = luminance.^(1/GAMMA) ;
if (level>0)
    LUT_params.gamma_table = gamma_table ;     % normal mode, default
%else econo mode
end
P.LUT_params = LUT_params ;


%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%     NOISE-FILTER  PARAMETERS     %%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% TEXTURE stimuli are filtered-noise patches. The amplitude spectrum of the
% filter is the same as the amplitude spectrum of a Gabor patch with
% spatial frequency dependent on the category. All filters are in a
% canonical (vertical) orientation but the patch will be rotated at a
% random angle inside CRFB2_do_trial.
% See CRFB2_make_filtnoise.m for details. Also utils/image/filter_img.m

filter_params.center_freq = center_freq ;   % cycles/degree
filter_params.bandwidth = bandwidth ;       % octaves

%-- Calculate the frequency sigma of the filter
% Consider a 1D Gaussian in the frequency domain, centered on Fc.
% Label the left point at half height X and the right point at half
% height Y. There are two constraints that must be met:
%  *  x+y = 2*Fc      % the central frequency is halfway b/n x and y
%  *  y/x = 2^b       % the ratio b/n y and x is b octaves
% The solution of this system of two equations with two unknowns is:
%  *  x = 2*Fc/((2^b)+1)
%  *  y = (2^b)*x
% The full width at half height FWHH:
%  *  FWHH = y-x = x((2^b)-1) = 2*Fc*((2^b)-1)/((2^b)+1)
% The sigma of the Gaussian is proportional to its FWHH
% See Table 2.1 in Norma Graham's "Visual Pattern Analyzers" (1989, p. 49).
% sigma = W / 2sqrt(2ln2) = 0.425 W ; W = sigma * 2sqrt(2ln2) = 2.35 sigma
%
% #####  freq_sigma = Fc * ((2^b)-1) / (((2^b)+1)*sqrt(2*log(2))   #####
%
% Watson & Eckert (1994), JOSA, 11, 496-505, Eq. 2 give a similar formula.
% Note that the sigma in the spatial domain is inversely proportional to
% the sigma in the frequency domain:  sigma_SP = 1 / 2*pi*freq_sigma
% Table 2.2 in Norma Graham's "Visual Pattern Analyzers" (1989:51).
k = 2^bandwidth ;
freq_FWHH = center_freq .* (2*(k-1)/(k+1)) ;  % cycles per degree
filter_params.freq_FWHH = freq_FWHH ;
W2sigma = 1 / (2*sqrt(2*log(2))) ;    % = 0.4247 = const, see GrahamN89
filter_params.W2sigma = W2sigma ;
freq_sigma = freq_FWHH .* W2sigma ;
filter_params.freq_sigma = freq_sigma ;

%-- Broad tuning in orientation
% All texture patches are generated in a canonical orientation.
% Then they are rotated to a random angle by CRFB2_do_trial.
center_orient = 0 ;   % degrees from vertical
orient_FWHH = 45 ;    % degrees
filter_params.center_orient = center_orient ;
filter_params.orient_FWHH = orient_FWHH ;

% Modeled after ~/work/models/PLModel1/frontend/RField_size.m
% Orient_FWHH is the same for all filters when expressed in polar
% coordinates in the frequency domain. In Cartesian coordinates, 
% it is proportional to the (center) spatial frequency.
orient_sigma = (orient_FWHH*W2sigma * pi/180) .* center_freq ;  % radians
filter_params.orient_sigma = orient_sigma ;

%-- Texture aperture -- see Mask Parameters below
aperture = [6.5 7.5] ;    % diameter of inner and outer rims, degrees
filter_params.aperture = aperture ;
assert(Mparams.aperture==mean(filter_params.aperture)) ;

img_size = ceil(aperture(2)*pix_deg) ;
filter_params.img_size = img_size ;
filter_size = 2^ceil(log2(max(img_size(:)))) ;  % next power of 2 up
filter_params.filter_size = filter_size ;

filter_params.noise_outcut = 10/3 ;   % studentize outliers +/-3.33*std

P.filter_params = filter_params ;


%-- Prepare a battery of filters for later calls to CRFB2_make_filtnoise.
Filter = zeros([filter_size filter_size, N_categ]) ;

[Fx,Fy] = fftmesh(filter_size) ;         % [-.5:+.5] cycles per pixel
Fx = Fx.*pix_deg ; Fy = Fy.*pix_deg ;    % cycles per degree (cpd)
% Fr = sqrt(Fx.^2 + Fy.^2) ;             % radius in Fourier space, cpd
if (level==9)    % debug mode
    P.Fx = Fx ;
    P.Fy = Fy ;
end

for k = 1:N_categ
    % The amplitude spectrum of a Gabor patch with vertical orientation
    % and spatial frequency center_freq consists of a pair of bivariate
    % Gaussians centered on [+center_freq,0] and [-center_freq,0].
    % Eq. 2.12 in Norma Graham's "Visual Pattern Analyzers" (1989,p.59) 
    Q1 = ((Fx-center_freq(k))./freq_sigma(k)).^2 + (Fy./orient_sigma(k)).^2 ;
    Q2 = ((Fx+center_freq(k))./freq_sigma(k)).^2 + (Fy./orient_sigma(k)).^2 ;
    % Ignore the multiplicative term as F will be normalized anyway
    F = exp(-0.5.*Q1) + exp(-0.5.*Q2) ;
    % Ensure mean(F(:).^2)==1.  See comments below
    norm_term = sqrt(mean(F(:).^2)) ;
    F = F ./ norm_term ;
    % Store and move on
    Filter(:,:,k) = F ;
end

P.Filter = Filter ;
%
% Note that the filter is normalized so that mean(F(:).^2)==1.
% This ensures that it preserves (on average) the stdandard deviation of
% the white-noise patch it is applied to. Proof: Let U be a 256x256
% white noise matrix in the spatial domain with mean(U(:))=0 and
% std(U(:))=sigma. Let F is a 256x256 filter in the Fourier domain
% with mean(F(:).^2)=1. Let V=filter_img(U,F) is the filtered image.
% By Parseval's Relation (Eq. 5.87 in "Signals and Systems," 1983)
%    mean(U(:).^2))==mean(abs(HU(:)).^2), where HU=fft2(U).
% But the expected fft2 of a white-noise patch is approx. constant:
%    abs(HU)==ones(256,256)*(sigma/N),  where N=256*256=65536.
% Hence the FFT of the filtered image basically takes the power spectrum
% of the filter (times a constant=sigma/N) and the phase spectrum of the
% original white-noise patch. Given that abs(HU) is flat:
%    abs(HV)=abs(F).*sigma , where HV=F.*HU
%    [Note that this works for white-noise patches only.]
% Since mean(abs(F(:)).^2)==1, we get mean(abs(HV(:)).^2)=sigma^2.
% Applying Parseval's Relation in reverse: mean(V(:).^2)=sigma^2.
% But mean(V(:))==0 because the DC component of F is set to 0.
% Hence the var(V)==mean(V.^2)==sigma^2   Q.E.D.

%-- Debugging tool: Plot the smallest and largest filters, etc.
if (level==9)     % debug mode
   fig_handle = fig_handle + 2 ;
   figure(fig_handle-1) ; clf(fig_handle-1) ;
   figure(fig_handle) ; clf(fig_handle) ;
   
   which_ones = [1 2 N_categ] ;  % which filters to plot
   n = length(which_ones) ;
   fx = fftshift(Fx(1,:)) ;
   fy = fftshift(Fy(:,1)) ;
   x = floor(filter_size/2) ;
   x = [1-x:x]./pix_deg ; %#ok<NBRAK>
   
   for k=1:n
       % Retrieve the relevant filter
       c = which_ones(k) ;
       F = Filter(:,:,c) ;
       
       % Plot the amplitude spectrum in Fourier domain:
       figure(fig_handle-1) ; 
       subplot(2,n,k) ;
       imagesc(fx,fy,fftshift(F)) ; axis image ;
       xlabel('Fx [cyc/deg]') ; ylabel('Fy') ;
       title(sprintf('fr.sig=%.2f, or.sig=%.2f',...
             freq_sigma(c),orient_sigma(c))) ;

       % ... and in cross-section along the Fx axis:
       subplot(2,n,k+n) ;
       plot(fx,fftshift(F(1,:))) ;
       xlabel('Fx [cyc/deg]') ; ylabel('Amplitude') ;
       grid on ;
       title(sprintf('center freq=%.2f',center_freq(c))) ;
       
       % IFFT of the filter should be a Gabor patch with vertical gratings
       figure(fig_handle) ;
       colormap gray ;
       % Do the inverse Fast Fourier Transform
       ifft_F = fftshift(real(ifft2(F))) ;
       % Plot the resulting Gabor patch
       subplot(2,n,k) ;
       imagesc(x,x,ifft_F) ; axis image ;
       title(sprintf('center freq=%.2f',center_freq(c))) ;
       
       % Plot sample filtered-noise patches
       subplot(2,n,k+n) ;
       FU = CRFB2_make_filtnoise(P,c) ;
       imagesc(x,x,FU,[black white]) ;
       % If CRFB2_make_filtnoise is not available, use this:
       % U=randn([filter_size filter_size]) ;
       % U = U./std(U(:)) ;
       % FU = filter_img(U,F) ;
       % m = max(abs(FU(:))) ;
       % imagesc(x,x,FU,[-m m]) ; 
       axis square ; axis xy ;
       title(sprintf('sd=%.2f',std(FU(:))));
   end
end


%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%   MASK PARAMETERS   %%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Make a circular mask with semitransparent ramp ^^\____/^^ at the edges.
% The mask is a texture with a transparency layer -- see DriftDemo2.m
% and FastMaskedNoiseDemo.m for examples.
% ALPHA varies from 0='fully transparent' to 255='fully opaque'
% The color of the mask is set to the background color of the screen
% and thus opaque regions are effectively erased.  Transparency requires
% that Alpha-Blending is enabled -- see Screen('BlendFunction',...).
% The footprint of the mask is inflated by COVERAGE_FACTOR to make
% sure that the corners of the square texture patch are fully masked
% even when the patch is rotated 45 degrees.
% To do: Consider putting a green fixation dot in the middle of the mask.
%        See FastMaskedNoiseDemo for an example
mask_params.coverage_factor = 1.42 ;  % sqrt(2)=1.4142 + a little reserve
mask_size = ceil(img_size*mask_params.coverage_factor) ;
mask_size = mask_size+mod(mask_size+1,2) ;   % must be an odd number
mask_params.mask_size = mask_size ;   % in pixels
R = make_grid(mask_size,1/pix_deg) ;  % distance from image center, degrees
mask_params.R = R ;
% Conversion from XY (degree) to IJ (pixel) coordinate origins.
% Usage:  ji = xy.*pix_deg + ji_origin ;
% Usage:  xy = (ji-ji_origin) ./ pix_deg ;
ji_origin = ceil(mask_size/2) ;  % an integer because MASK_SIZE is odd
mask_params.ji_origin = ji_origin ;

[X,Y] = meshgrid(R,R) ;     %  Cartesian coordinates, origin in the middle
RR = sqrt(X.^2 + Y.^2) ;    % [mask_size x mask_size], in degrees
Rmin = filter_params.aperture(1) / 2 ;
Rmax = filter_params.aperture(2) / 2 ;
alpha = (RR - Rmin) ./ (Rmax - Rmin) ;
alpha = max(0.0,min(1.0,alpha)) ;     % radially sym. ramp  _____/^^^^^

if (level==9)     % debug mode
    %- Export meshgrid for inspection
    mask_params.X = X ;
    mask_params.Y = Y ;
    mask_params.alpha = alpha ;
    %- Plot the mask as an image
    fig_handle = fig_handle + 1 ;
    figure(fig_handle) ; clf(fig_handle) ;
    subplot(2,2,1) ;
    imagesc(R,R,alpha,[0 1]) ; axis square ;
    colormap(flipud(gray)) ; colorbar ;
    xlabel('space [degrees]') ; ylabel('space [degrees]') ;
    title('Transparency') ;
    %- Plot in cross-section
    subplot(2,2,2) ;
    plot(R,alpha(ji_origin,:)) ; axis([-8 8 -.1 1.1]) ; grid on ;
    xlabel('Radius [deg]') ; ylabel('Alpha (=transparency)') ;
end

% The mask consists of a monochrome luminance plane and an alpha plane.
% Usage: masktex = Screen('MakeTexture',whichScreen,P.mask) ;
%        Screen('DrawTexture',winPtr,masktex,maskszRect,maskposRect) ;
mask = repmat(LUT_params.backgr,mask_size,mask_size) ;
mask(:,:,2) = alpha.*LUT_params.span ;
maskszRect = [0 0 mask_size mask_size] ;
mask_params.maskszRect = maskszRect ;
P.mask_params = mask_params ;
P.mask = mask ;


%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%     WINDOW PARAMETERS     %%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Everything is presented in a square window in the middle of the screen.
% RDC coordinates are relative to the midpoint of the window.

win_params.whichScreen = whichScreen ;
if (deployedp)
    winRect = SetRect(0,0,1024,768) ;   % on iScope1, for data collection
    % as per Screen.OpenWindow
elseif (strcmpi(host,'vega'))
    winRect = SetRect(0,0,1680,1050) ;  % on Vega, for program development only
else
    winRect = SetRect(0,0,1024,768) ;
end
win_params.winRect = winRect ;
[Xcenter,Ycenter] = RectCenter(winRect) ;     % 512,384
win_params.XY_center = [Xcenter,Ycenter] ;
imgszRect = SetRect(0,0,img_size,img_size) ;  % [left,top,rgt,bot]
win_params.imgszRect = imgszRect ;
win_params.imgposRect = CenterRectOnPoint(imgszRect,Xcenter,Ycenter) ;
win_params.maskszRect = mask_params.maskszRect ;
win_params.maskposRect = CenterRectOnPoint(maskszRect,Xcenter,Ycenter) ;

%-- Fixation dots are rendered via FillOval, which requires a rectangle:
% The fixation dot is drawn via Screen('FillOval') in CRFB2_static_display
% Usage: Screen('FillOval',winPtr,foregr,fixdotRect) ;
win_params.fix_dot_diam = 0.2 ;   % degrees of visual angle
fix_dot_diam_in_pixels = win_params.fix_dot_diam * pix_deg ;
fixdotRect = SetRect(0,0,fix_dot_diam_in_pixels,fix_dot_diam_in_pixels) ;
fixdotRect = CenterRectOnPoint(fixdotRect,Xcenter,Ycenter) ;
win_params.fixdotRect = fixdotRect ;  % used in CRFB2_static_display

%-- Feedback-related
% See bonus_params below and ML_display_message

P.win_params = win_params ;


%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%    TIMING  PARAMETERS     %%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% All times are expressed in seconds relative to beginning of trial
% 0.000 -- Alert beep onset marks the beginning of trial
% 0.020 -- Alert beep offset
% 0.500 -- Stimulus onset
% Indefinite response window
% X     -- Response typed in; Feedback onset
% X+1.100 -- Feedback offset, same as in Feedback1 experiment

tim.ITI = 0.500 ;   % inter-trial interval, seconds, same as in Feedback1

tim.alert_onset = 0.000 ;  % by definition
tim.alert_duration = 0.020 ;
tim.stimulus_onset = 0.500 ;
tim.feedback_duration = 1.100 ;

% Target_play is needed when stimuli have a prescribed duration that must
% be verified by CRFB2_do_trial (exit code EC_INSTRUM). Not the case here.
%tim.target_play = Mparams.max_duration_seconds ;

% To discourage speed/accuracy tradeoffs, penalize the subject if the
% response time drops below 1000 msec. (Feedback1 data indicate that less
% than 5% of the RTs are faster than 1000 msec in a distance rating task.)
% The faster the subject goes, the longer idle wait there is.
% Also, there is a hard minimum (500 msec) below which the trial is
% repeated. See CRFB2_do_trial for details.
tim.RT_penalty = 1.000 ;    % penalize response times < 1000 msec
tim.toofast_crit = 0.500 ;  % trigger message "Please slow down... :-)"
tim.toofast_penalty = 1.500 ; % wait on "Please slow down... :-)"

P.time_params = tim ;


%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%    RESPONSE  PARAMETERS     %%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

resp_params.keys = ...     % type 'edit KbName' for details
    {{'1!' '1'} ...        % category 1: main '1!' key or keypad '1' key
    ,{'2@' '2'} ...
    ,{'3#' '3'} ...
    ,{'4$' '4'} ...
    ,{'5%' '5'} ...
    ,{'6^' '6'} ...
    ,{'7&' '7'} ...
    } ;
assert(length(resp_params.keys)==N_categ) ;

% used by CRFB2_dummy_response
resp_params.dummy_error = [ -2  -1   0   1   2] ;
resp_params.dummy_Perr  = [.10 .15 .50 .15 .10] ;
resp_params.dummy_cPerr = cumsum(resp_params.dummy_Perr) ;
assert(resp_params.dummy_cPerr(end)==1.0) ;

resp_params.dummy_RT = [1000 200] ;       % [mean std] in milliseconds

P.resp_params = resp_params ;


%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%           SOUNDS          %%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if (level>0 && PsychToolbox_avail)
  %- Technicalities: See ..../PsychDemos/BasicSoundOutputDemo.m
  samplingRate = 8192 ;   % Hz
  P.sounds.samplingRate = samplingRate ;
  P.sounds.channels = 1;  % One channel only -> Mono sound
  % Sounds are row vectors with one row per channel:
  ramp = [1:100]./100 ; %#ok<NBRAK>
  
  %- Soft brief alert beep
  s = MakeBeep(500,tim.alert_duration,samplingRate) ;   % 500 Hz, 20 msec
  env = ones(size(s)) ;
  env(1:100) = ramp ; env(end-99:end) = fliplr(ramp) ;
  P.sounds.alert  = s .* env .* 0.5 ;
  
  %- Loud buzz on wrong responses -- not used in this experiment
  %s = [MakeBeep(400,.030,samplingRate) , ...
  %     MakeBeep(300,.120,samplingRate)] ;     % 150 msec total
  %env = ones(size(s)) ;
  %env(1:100) = ramp ; env(end-99:end) = fliplr(ramp) ;
  %P.sounds.wrong = s .* env ;
  
  %- Long loud buzz on invalid keys -- not used in this experiment
  %s = [MakeBeep(400,.050,samplingRate) , ...
  %     MakeBeep(350,.200,samplingRate)] ;     % 250 msec total
  %env = ones(size(s)) ;
  %env(1:100) = ramp ; env(end-99:end) = fliplr(ramp) ;
  %P.sounds.badkey = s .* env ;
end


%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%      BONUS PARAMETERS      %%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% To boost motivation, the subject scores bonus points for each
% response that perfectly matches the correct category.
% There is no penalty for incorrect responses.
% In feedback blocks, the updated bonus is shown on each trial.
% In no-feedback blocks, the accumulated bonus is shown only at the end.
%
% Usage:  new_bonus = bonus_params.policy(correct+1)
bonus_params.policy = [0 +1 ] ;  % [incorrect, correct]
bonus_params.init_bonus = 20 ;   % avoid negatives during demo

% Text-related
bonus_params.TextSize = 24 ;
bonus_params.TextFont = 'Monaco' ; % fixed-width font
bonus_params.fdbk_TextSize = 180 ;

% Pen position for ML_display_bonus.m
bonus_params.bonus_X = Xcenter-30 ;             % pixels
%bonus_Y = (1+max(filter_params.aperture)/2) ;  % degrees of visual angle
%bonus_Y = ceil(bonus_Y*pix_deg) ;              % pixels
bonus_Y = ceil(mask_size/2) + bonus_params.TextSize + 1 ;  % pixels
bonus_params.bonus_Y = Ycenter-bonus_Y ;

% Pen position for ML_display_message.m
bonus_params.message_X = 100 ;   % pixels from left edge of screen
bonus_params.message_Y = 240 ;   % pixels from top edge of screen

% Pen position for displaying feedback within CRFB2_do_trial
bonus_params.fdbk_X = Xcenter - bonus_params.fdbk_TextSize/3 ;
bonus_params.fdbk_Y = Ycenter - bonus_params.fdbk_TextSize/1.5 ;

P.bonus_params = bonus_params ;


%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%    FILE NAMES AND PATHS    %%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

root = CRFeedback2_pathstr ;
filenames.root_path = CRFeedback2_pathstr ;
filenames.data_path = fullfile(root,'data') ;
% The sbj database and log are kept separate on different host computers
if (strcmpi(host,'iscope1'))
    filenames.DB_fname = 'CRFB2_subjects_iScope1' ;
    filenames.log_fname = 'CRFeedback2_iScope1.log' ;
elseif (strcmpi(host,'iscope2'))
    filenames.DB_fname = 'CRFB2_subjects_iScope2' ;
    filenames.log_fname = 'CRFeedback2_iScope2.log' ;
elseif (strcmpi(host,'iscope3'))
    filenames.DB_fname = 'CRFB2_subjects_iScope3' ;
    filenames.log_fname = 'CRFeedback2_iScope3.log' ;
else   % Software development
    filenames.DB_fname = 'CRFB2_subjects_debug' ;
    filenames.log_fname = 'CRFeedback2_debug.log' ;
end
P.filenames = filenames ;


%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%     ENUMERATION TYPES      %%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%-- Dry_run enumeration type. Begins at zero
dry_run_enum = {'HUMAN_DATA', 'EXTENDED_DRY_RUN', 'QUICK_DRY_RUN'} ;
enum.dry_run_enum = dry_run_enum ;
for k = 1:length(dry_run_enum)
    enum.(dry_run_enum{k}) = k-1 ;   % enum.HUMAN_DATA = 0; etc.
end

%-- CRFB2_do_trial exitcode enumeration type. Begins at zero.
exitcode_enum = {'EC_OK', 'EC_BAD_KEY', 'EC_FAST_KEY', 'EC_INSTRUM'} ;
enum.exitcode_enum = exitcode_enum ;
for k = 1:length(exitcode_enum)
    enum.(exitcode_enum{k}) = k-1 ;   % enum.EC_OK = 0; etc.
end

%-- Request data file herald (see CRFB2_file_herald.m)
% herald_request is a cell array with two columns. The left item must be
% one of fields_to_log, the right item is the corresponding enum type:
herald_request = { ...
    'task',      'task_enum'     ; ...
    'schedule',  'block_enum'    ; ...
    'dry_run',   'dry_run_enum'  ; ...
    'exitcode',  'exitcode_enum' ; ...
    } ;
enum.herald_request = herald_request ;

P.enum = enum ;

%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%  DATA FIELDS TO LOG. FORMATTING  %%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Usage:  fnames = fields_to_log(:,1) ; formats = fields_to_log(:,2) ;
% All field names are case sensitive. See ML_write_data.m
% The order of fields_to_log determines the order of columns in the data
% file. It must coincide with P.data.indices. See CRFB2_file_herald.m
% P.data.indices may contain extra fields such as 'old_bonus' that are
% not preserved in the data file. They come last in the list.

fields_to_log = {...
    'sbj',         '%03d' ;... 
    'group',       '%d' ; ...
    'session',     '%d' ; ...
    'blk',         '%02d' ; ...
    'schedule',    '%d' ; ...          % see P.enum.block_enum
%    'demop',       '%d' ; ...
    'expT',        '%04d' ; ...
    'blk_tr',      '%03d' ; ...
    'dry_run',     '%d' ; ...          % see P.enum.dry_run_enum
    'task',        '%d' ; ...          % see P.enum.task_enum
    'categ',       '%d' ; ...
    'orient',      '%+04d' ; ...
    'feedbackp',   '%d' ; ...
    'resp',        '%d' ; ...
    'corr',        '%d' ; ...
    'RT' ,         '%05d' ; ...
    'bonus' ,      '%04d' ; ...
    'exitcode',    '%d' ; ...          % see P.enum.exitcode_enum
    'clock',       '%09.3f' ; ...
    'spare_ITI',   '%+05d' ; ...
    'delta_msec',  '%02d' ; ...
    'frame_count', '%03d' ; ...
    'stim_msec',   '%05d' ; ...
    };
P.data.fields_to_log = fields_to_log ;


fnames  = fields_to_log(:,1) ;   % cell(N_columns,1)
N_fields = length(fnames) ;
for k = 1:N_fields
    I.(fnames{k}) = k ;
end

%- The fields below are not preserved in the data file
I.old_bonus = N_fields+1 ;
P.data.indices = I;
array_size = length(fieldnames(I)) ;
P.data.array_size = array_size;


%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%    VALUES STORED FOR CLEANUP    %%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

cleanup_vals.pwd = pwd ;
cleanup_vals.whichScreen = whichScreen ;
if (PsychToolbox_avail)
    cleanup_vals.gamma_table = Screen('ReadNormalizedGammaTable',whichScreen) ;
end
% More fields will be added by CRFB2_prepare ...
P.cleanup_vals = cleanup_vals ;

%--- Return P
%%%%% End of file CRFB2_params.m
