%% fit_ANCHOR_ARL2_part1.m
%
% Fit the ANCHOR2 model to the group ARL2 profiles of the Fdbk1 Experiment.
%
% The main tool is .../work/models/anchor/fit_SSE/anchor_ARL2.m
%
%  File: work/CMUExper/fdbk1/data/fit_ANCHOR_ARL2_part1.m
%  Date: 2007-08-17
%  Alexander Petrov, http://alexpetrov.com


%% Load the data
clear all
cd(fullfile(work_pathstr,'CMUExper','fdbk1','data')) ;
load('S2.mat') ;
ARL2 = [S2(:).ARL2] ;    % 9x55
gr=[S2(:).group]' ;
ugr = unique(gr) ;
N_groups = length(ugr) ;
ugr_descr = {'U1' 'L1' 'H1', 'L2' 'H2' } ;   % [1 2 3, 5 6]

%% Collapse across groups
grARL2 = zeros(size(ARL2,1),N_groups) ;   % 9x5
gr_idx = cell(1,N_groups) ;
for k = 1:N_groups
    gr_idx{k} = find(gr==ugr(k))' ;
    grARL2(:,k) = mean(ARL2(:,gr_idx{k}),2) ;
end
grARL2

%% Fit INST on the same stimulus sequences, 11 per group
stim = [S2(:).dist] ;   % 476x55
fdbk = [S2(:).fdbk] ;

%% Search parameters
% Same as those used for fitting the INST model to the same profiles.
% See ./fit_INST_ARL2_part1.m for details
fminconp = 1 ;
flags = [0 1 1 0 0 0 0 0] ;   % [perc_k mem_k temper hist cutoff xx xx c_ratio]
Sparams = anchor_search_params2(flags,fminconp) ;
optns = optimset(Sparams.optns,'TolX',1e-2,'Display','off') ;
Sparams.optns = optns ;
Sparams.target_vals = grARL2
H = .040 ; Sparams.params.history = H ;   % for the time being
Sparams.params

%% "Chance event" sets
% The model is fitted using the deterministic version PROTOANCHOR_CHEV
% that uses previously cached values of all randomly generated values.
% More than one such "chev" sets is used to evaluate its impact on the fit.
N_chevs = 2 ;

% Pack into "model data" structures suitable for passing as arguments.
Mdata = cell(1,N_chevs) ;
for k = 1:N_chevs
    Mdata{k}.stim = stim ;
    Mdata{k}.fdbk = fdbk ;
    Mdata{k}.group_idx = gr_idx ;
    %Mdata{k}.params = Sparams.params ;    % will be set by SUMSQERR
    Mdata{k}.chev = make_anchor_chev(size(stim,1),size(stim,2),7) ;
    Mdata{k}.ARL_params = Sparams.ARL_params ;
end

%% Cutoff parameter grid
cutoff_levels = [.50 .55 .60 .65 .70 .75 .80 .85 .90]' ;  % c- in categ_sz units
N_cutoff_levels = length(cutoff_levels) ;
c_ratio_levels = [.5 .6 .65 .7 .75 .8 .9] ;         % c-/c+ ratio
N_c_ratio_levels = length(c_ratio_levels) ;

%% Main work
% A grid search over 9x7x2 cells takes 13.5 hours on a fast machine.
% Therefore, the results will only be computed once and cached to disk.
fname = 'grid_search_ANCHOR_ARL2_part1.mat' ;
if (file_exists(fname))
    load(fname) ;
else
  grid_SSE = zeros([N_cutoff_levels,N_c_ratio_levels,N_chevs]) ;
  grid_optX = cell([N_cutoff_levels,N_c_ratio_levels,N_chevs]) ;
  grid_det = cell([N_cutoff_levels,N_c_ratio_levels,N_chevs]) ;
  tic
  for k1 = 1:N_cutoff_levels
      c = cutoff_levels(k1) ;
      fprintf('\n cutoff %4.2f: ',c) ;
      for k2 = 1:N_c_ratio_levels
          cr = c_ratio_levels(k2) ;
          cutoffs = c .* [-3 -1 cr 3*cr] ;
          Sparams.params.cutoffs = cutoffs ;
          for k = 1:N_chevs
              [par,SSE,optX,det] = paramsearch(Mdata{k},Sparams) ;  % <-- Sic!
              grid_SSE(k1,k2,k) = SSE ;
              grid_optX{k1,k2,k} = optX ;
              grid_det{k1,k2,k} = det ;
              fprintf('%5.3f ',SSE) ;
              save(fname,'grid_SSE','grid_optX','grid_det','Sparams',...
                   'cutoff_levels','c_ratio_levels') ;
          end
          fprintf(', ') ;
      end
  end
  fprintf('\n') ;
  toc
end % if file_exists ...

%% Plot the SSE
SSE = mean(grid_SSE,3)   % average across the chev's
[X,Y] = meshgrid(c_ratio_levels,cutoff_levels) ; 
contourf(X,Y,SSE) ; colorbar ; title('SSE') ;
xlabel('c-/c+ ratio') ; ylabel('c- cutoff') ;

%% Plot the memory noise parameter
mem_k = zeros(size(grid_optX)) ;
temper = zeros(size(grid_optX)) ;
for k1 = 1:N_cutoff_levels
    for k2 = 1:N_c_ratio_levels
        for k = 1:N_chevs
            mem_k(k1,k2,k) = grid_optX{k1,k2,k}(1) / 10 ;
            temper(k1,k2,k) = grid_optX{k1,k2,k}(2) / 10 ;
        end
    end
end

mem_k = mean(mem_k,3)
contourf(X,Y,mem_k) ; colorbar ; title('Memory noise') ;
xlabel('c-/c+ ratio') ; ylabel('c- cutoff') ;

%% Plot the temperature parameter
temper = mean(temper,3)
contourf(X,Y,temper) ; colorbar ; title('Softmax temperature') ;
xlabel('c-/c+ ratio') ; ylabel('c- cutoff') ;

%% Interim conclusion
%  Fix c_ratio = .75 once and for all
%  Explore  cutoff in the range [.55 .60 .65 .70]
%  Explore  history in the range  [.020 .030 .040 .050 .060]
%  Initial search value for mem_k = .085
%  Initial search value for temper = .030
%
% The fitting effort continues in fit_ANCHOR_ARL2_part2.m

%% Clean up
clear k k1 k2 X Y ;
