function  [dist,categ,fdbk] = make_fdbk1_stimuli(group,N_runs)

% MAKE_FDBK1_STIMULI -- Generate a stimulus sequence for the FDBK1 Expmt.
%   
% [dist,categ,fdbk] = make_fdbk1_stimuli(group,N_runs)
%
% GROUP is an integer from 1 to 6. See CMUExper/programs/fdbk1/Fdbk_exp1.lisp
%  (1 '(:init :U-fb :U-no :U-fb :U-no))      ; (0 1 2 1 2)  ; Group U
%  (2 '(:init :P-fb :P-no :P-fb :P-no))      ; (0 3 4 3 4)  ; Group P
%  (3 '(:init :N-fb :N-no :N-fb :N-no))      ; (0 5 6 5 6)  ; Group N
%  (4 '(:init :U-no :U-fb :U-no :U-fb))      ; (0 2 1 2 1)  ; Group U1
%  (5 '(:init :P-no :P-fb :P-no :P-fb))      ; (0 4 3 4 3)  ; Group P1
%  (6 '(:init :N-no :N-fb :N-no :N-fb))      ; (0 6 5 6 5)  ; Group N1
% N_RUNS specifies how many sequences to generate. Default = 10.
% DIST, CATEG, and FDBK are 476-by-N_RUNS matrices.
%
% See also ANCHOR2, PROTOANCHOR, ASSIMIL_PROFILES.

% Original coding by Alex Petrov, Ohio State University
% $Revision: 1.0 $  $Date: 2007-05-29 $

if (nargin<2)  N_runs = 10 ; end   % default

%-- FDBK1 expmt params. See work/CMUExper/programs/fdbk1/Fdbk_exp1.lisp
categ2dist = [420 460 500 540 580 620 660]' ; % *7-categ-1-stim-each-40-pix-apart*

U_categ = [1 1 1 1  2 2 2 2  3 3 3 3  4 4 4 4  5 5 5 5  6 6 6 6  7 7 7 7]' ;
N_categ = [1  2 2  3 3 3  4 4 4 4  5 5 5 5 5  6 6 6 6 6 6  7 7 7 7 7 7 7]' ;
P_categ = [1 1 1 1 1 1 1  2 2 2 2 2 2  3 3 3 3 3  4 4 4 4  5 5 5  6 6  7]' ;

N_trials = 28 + 4*4*28 ;

fdbk_first = ones(N_trials,1) ;
fdbk_first(28+1*4*28+[1:4*28]) = 0 ;
fdbk_first(28+3*4*28+[1:4*28]) = 0 ;
fdbk_second = ones(N_trials,1) ;
fdbk_second(28+0*4*28+[1:4*28]) = 0 ;
fdbk_second(28+2*4*28+[1:4*28]) = 0 ;

%-- Main loop
categ = zeros(N_trials,N_runs) ;
switch (group)
    case {1 4}  % uniform
        for k = 1:N_runs  categ(:,k) = make_block(U_categ,U_categ) ; end
    case {2 5}  % positively skewed
        for k = 1:N_runs  categ(:,k) = make_block(P_categ,U_categ) ; end
    case {3 6}  % positively skewed
        for k = 1:N_runs  categ(:,k) = make_block(N_categ,U_categ) ; end
    otherwise
        error('Invalid group code (%d).',group) ;
end

dist = categ2dist(categ) ;

fdbk = categ ;
switch (group)
    case {1 2 3}
        fdbk = categ .* repmat(fdbk_first,1,N_runs) ;
    case {4 5 6}
        fdbk = categ .* repmat(fdbk_second,1,N_runs) ;
end

%---  Return DIST, CATEG, and FDBK
%%%%  End of main function


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%
function  categ1 = make_block(sched,init_sched)

% *number-of-trials-per-mini-block* = 28
% *number-of-mini-blocks-per-block* = 4

idx = [randperm(28) randperm(28) randperm(28) randperm(28) , ...  % blk 1
       randperm(28) randperm(28) randperm(28) randperm(28) , ...  % blk 2
       randperm(28) randperm(28) randperm(28) randperm(28) , ...  % blk 3
       randperm(28) randperm(28) randperm(28) randperm(28) ]' ;   % blk 4

categ1 = [init_sched(randperm(28)') ; sched(idx) ] ;

%%%% End of subfunction MAKE_BLOCK

%%%%% End of file MAKE_FDBK1_STIMULI.M
