function  instances = default_instances(range,N)

% DEFAULT_INSTANCES   Initial instances for the PROTOANCHOR model.
%   
%   INSTANCES = DEFAULT_INSTANCES
%   INSTANCES = DEFAULT_INSTANCES(RANGE)
%   INSTANCES = DEFAULT_INSTANCES(RANGE,N)
%
%   Constructs a Nx3 matrix suitable for filling the INSTANCES slot of
%   the PARAMS structure for the PROTOANCHOR model.
%   RANGE is a two-element vector specifying the lowest and highest 
%   anchor magnitude. When not supplied, it defaults to [0.420 0.660].
%   N specifies the number of instances. Default 7.
%
%   The INSTANCES matrix has a row describing each anchor.
%   Each row has six columns with the following semantics:
%    * magnitude -- a real number between 0 and 1
%    * response  -- a scale value between 1 and N
%    * time_of_creation -- a real number, default -10, see BASE_ACT
%
%   Example: instances = default_instances returns the following matrix:
%    0.4200    1.0000  -10.0000
%    0.4600    2.0000  -10.0000
%    0.5000    3.0000  -10.0000
%    0.5400    4.0000  -10.0000
%    0.5800    5.0000  -10.0000
%    0.6200    6.0000  -10.0000
%    0.6600    7.0000  -10.0000
%
%   See also PROTOANCHOR_PARAMS, DEFAULT_ANCHORS.

% Original coding by Alex Petrov, Ohio State University
% $Revision: 1.0 $  $Date: 2007-04-06 15:16 $
%
% Copyright (c) Alexander Petrov 1999-2007, http://alexpetrov.com
% Please read the LICENSE and NO WARRANTY statement in anchor_license.m

%--- Decipher input arguments
if (nargin < 1)
  range = [.420 .660] ;
  N = 7 ;
elseif (nargin < 2)
  N = 7 ;
end

min_mag = range(1) ;
max_mag = range(2) ;
step = (max_mag - min_mag) / (N - 1) ;

%---  Build the matrix
magnitudes = [min_mag : step : max_mag]' ;
responses  = [1:N]' ;
time_of_creation = repmat(-10,N,1) ;    % -10 for all instances

instances = [magnitudes responses time_of_creation] ;

%%%%% End of file DEFAULT_INSTANCES.M
