function  E_resp = protoanchor_expect(Mdata)

% PROTOANCHOR_EXPECT -- PROTOANCHOR_CHEV returning response *expectations*
%   
%  E_resp = protoanchor_expect(Mdata)
%
% Instead of a discrete response, a probabilistic mixture of responses
% is produced on each trial in an effort to smooth out the derivative
% and facilitate LSQNONLIN.
%
% Mdata is a structure with fields:
%  * stim -- [N_trials,N_runs] matrix of stimulus intensities
%  * fdbk -- can be []
%  * params -- see PROTOANCHOR_PARAMS
%  * chev -- see MAKE_PROTOANCHOR_CHEV
%
% Example (20 runs in Group P1 of Fdbk1 experiment):
%  Mparams = protoanchor_params([1:7]) ;
%  Mparams.history = .040 ; Mparams.cutoffs = .80.*[-3 -1 1 3] ;
%  N_runs = 20 ; [stim,foo,fdbk] = make_fdbk1_stimuli(2,N_runs) ;
%  Mdata.stim = stim ; Mdata.fdbk = fdbk ; Mdata.params = Mparams ;
%  Mdata.chev = make_protoanchor_chev(size(stim,1),N_runs) ;
%  resp1=protoanchor_chev(Mdata); [arl1,ARL_params]=fdbk1_ARL2(stim,resp1);
%  resp2=protoanchor_expect(Mdata); arl2=fdbk1_ARL2(stim,resp2);
%  plot([mean(arl1,2) mean(arl2,2)],'o-');legend('discrete','smoothed');
%
% See also PROTOANCHOR, PROTOANCHOR_CHEV, PROTOANCHOR_ARL2, PARAMSEARCH.

% Original coding by Alex Petrov, Ohio State University
% $Revision: 1.0 $  $Date: 2007-08-15 $
%
% Copyright (c) Alexander Petrov 1999-2007, http://alexpetrov.com
% Please read the LICENSE and NO WARRANTY statement in anchor_license.m

%---  Decipher Mdata
stimuli = Mdata.stim ;
fdbk = Mdata.fdbk ;
params = Mdata.params ;
chev_target_noise = 1+ Mdata.chev.target_mag_noise .* params.perc_k ;
chev_inst_noise = Mdata.chev.inst_mag_noise ;
chev_softmax1 = Mdata.chev.softmax1_chev ;

[N_trials,N_sbj] = size(stimuli) ;
if (isempty(fdbk))
  fdbk = zeros(N_trials,N_sbj) ;   % no feedback
elseif (any(size(fdbk)~=[N_trials,N_sbj]))
  error('STIMULI and FEEDBACK must have the same shape.') ;
end

%---  Decipher PARAMS
inter_trial_interval = params.ITI ;
SM_conv = params.SM_conv ;     % stimulus continuum --> magnitude continuum
N_cat = params.N_cat ;         % number of categories == number of anchors
category_sz = params.cat_sz ;  % width of one response category
cutoffs = params.cutoffs * category_sz ;  % [-2 -1 +1 +2]
max_incrmt = length(cutoffs) / 2 ;
%perc_k = params.perc_k ;       % noise in the target magnitude
mem_k = params.mem_k ;         % noise in anchor magnitudes
temper = params.temper ;       % temperature for anchor softmax
H = params.history ;           % history factor (=1/mismatch_penalty)
minus_decay = -params.decay ;  % activation decay rate
%alpha = params.alpha ;         % competitive learning rate
%one_alpha = 1 - alpha ;
N_seed_inst = size(params.instances,1) ;   % number of seed instances

%---  Convert physical stimuli into internal magnitudes ; avoid MAG_SCALE
%  Only LINEAR scales are supported, that is, Stevens law with n=1.
mags = SM_conv .* stimuli ;          % Stevens law
mags = mags .* chev_target_noise ;   % Ekman's law, perc_k multiplied above

%---  Main work
responses = zeros(N_trials,N_sbj) ;
E_resp = zeros(N_trials,N_sbj) ;
for k = 1:N_sbj
  %-- Re-initialize
  inst_mags = [params.instances(:,1) ; zeros(N_trials,1) ] ;
  inst_resps = [params.instances(:,2) ; zeros(N_trials,1) ] ;
  time_of_creation = [params.instances(:,3) ; zeros(N_trials,1) ] ;
  M = N_seed_inst ;  % current number of active entries in INST_xxxx
  curr_time = 0 ;
  %-- Main loop
  for tr = 1:N_trials
    target_mag = mags(tr,k) ;
    curr_time = curr_time + inter_trial_interval ;
    %- Calculate the base-level activations of the anchors
    T = curr_time - time_of_creation(1:M) ;  % lifetimes of the instances
    B = minus_decay .* log(T) ;      % base-level activation
    %- Add multiplicative noise to the anchor magnitudes
    nn = 1 + mem_k .* chev_inst_noise{tr}(1:M,k) ;
    noisy_inst_mags = inst_mags(1:M) .* nn ;
    %- Form match scores:  Si = -|M-Ai| + H*Bi
    mismatch = abs(noisy_inst_mags - target_mag) ;
    scores = H.*B - mismatch ;
    %- Retrieve an instance according to the Boltzmann equation
    scores = scores - min(scores) ;  % make all scores have the same sign
    u = chev_softmax1(tr,k) ;        % cached uniform random deviate
    [win_score,curr_inst_idx] = softmax1_chev(scores,temper,u) ;
    curr_inst_resp = inst_resps(curr_inst_idx) ;
    inst_mag = noisy_inst_mags(curr_inst_idx) ;
    %- Determine the correction
    delta = target_mag - inst_mag ;
    curr_incrmt = max_incrmt - sum(cutoffs > delta) ;
    %- Correct the instance response by the incrmt; incrmt can be negative
    target_resp = curr_inst_resp + curr_incrmt ;
    target_resp = max(1,min(N_cat,target_resp)) ;  % clip within bounds
    %- Register response
    responses(tr,k) = target_resp ;
    %- Store the instance, with or without feedback
    stored_resp = fdbk(tr,k) ;
    if (stored_resp==0)             % if no external feedback
      stored_resp = target_resp ;   % rehearse own response
    end
    %fprintf('trial %d, storing instance %d.\n',tr,stored_resp) ;
    %####  Expectations
    %#- Retrieval probability for each instance
    P = exp(scores./temper) ;   % same dimension as noisy_inst_mags  
    P = P ./ sum(P) ;           % see SOFTMAX1_CHEV
    %#- Column-vector of corrections
    deltas = target_mag - noisy_inst_mags ;
    bits = repmat(cutoffs,length(noisy_inst_mags),1) > repmat(deltas,1,4) ;
    incrmts = max_incrmt - sum(bits,2) ;
    %#- Vector of corrected responses
    resps = inst_resps(1:M) + incrmts ;
    resps = max(1,min(N_cat,resps)) ;   % clip within bounds
    %#- Register the expected response
    E_resp(tr,k) = resps' * P ;
    %- Move on
    M = M + 1 ;
    inst_mags(M) = target_mag ;
    inst_resps(M) = stored_resp ;
    time_of_creation(M) = curr_time ;
  end % of tr=1:N_trials
end % of k=1:N_sbj

%--- Return E_RESP
%%%%% End of file PROTOANCHOR_EXPECT.M
