function  [responses,l] = protoanchor_log(stimuli,params,fdbk)

% PROTOANCHOR_LOG  -- Log-keeping version of the PROTOANCHOR model.
%   
%  [responses,l] = protoanchor_log(stimuli,params)
%  [responses,l] = protoanchor_log(stimuli,params,fdbk)
%
%   STIMULI, PARAMS, and RESPONSES are exactly the same as in PROTOANCHOR.
%   The log L is a structure array with the following fields:
%    1. target_mag -- target magnitude, [N_trials,1]
%    2. winner -- the index of the retrieved instance, [N_trials,1]
%    3. winner_mag -- the (noisy) magnitude of the winner, [N_trials,1]
%    4. winner_resp -- the label of the winner, [N_trials,1]
%    5. incrmt -- correction {-2,-1,0,+1,+2}, [N_trials,1]
%    6. memory -- memory pool at the end of the session
%
%  See also PROTOANCHOR, ANCHOR2LOG, PROTOANCHOR_PARAMS.

% Original coding by Alex Petrov, Ohio State University
% $Revision: 1.0 $  $Date: 2007-04-06 18:02 $
%
% Copyright (c) Alexander Petrov 1999-2007, http://alexpetrov.com
% Please read the LICENSE and NO WARRANTY statement in anchor_license.m

%---  Handle arguments
if (nargin < 2)
  error('At least two input arguments expected: STIMULI and PARAMS.') ;
end
[N_trials,N_sbj] = size(stimuli) ;
if (nargin < 3)
  fdbk = zeros(N_trials,N_sbj) ;   % no feedback
elseif (any(size(fdbk)~=[N_trials,N_sbj]))
  error('STIMULI and FEEDBACK must have the same shape.') ;
end

%---  Decipher PARAMS
inter_trial_interval = params.ITI ;
SM_conv = params.SM_conv ;     % stimulus continuum --> magnitude continuum
N_cat = params.N_cat ;         % number of categories == number of anchors
category_sz = params.cat_sz ;  % width of one response category
cutoffs = params.cutoffs * category_sz ;  % [-2 -1 +1 +2]
max_incrmt = length(cutoffs) / 2 ;
perc_k = params.perc_k ;       % noise in the target magnitude
mem_k = params.mem_k ;         % noise in anchor magnitudes
temper = params.temper ;       % temperature for anchor softmax
H = params.history ;           % history factor (=1/mismatch_penalty)
minus_decay = -params.decay ;  % activation decay rate
alpha = params.alpha ;         % competitive learning rate
one_alpha = 1 - alpha ;
N_seed_inst = size(params.instances,1) ;   % number of seed instances

%---  Convert physical stimuli into internal magnitudes ; avoid MAG_SCALE
%  Only LINEAR scales are supported, that is, Stevens law with n=1.
mags = SM_conv .* stimuli ;                            % Stevens law
mags = mags .* normrnd(ones(N_trials,N_sbj),perc_k) ;  % Ekman's law

%---  Main work
responses = zeros(N_trials,N_sbj) ;
for k = 1:N_sbj
  %-- Allocate log structure
  l(k).target_mag = mags(:,k) ;
  l(k).winner     = zeros(N_trials,1) ;
  l(k).winner_mag = zeros(N_trials,1) ;
  l(k).winner_resp= zeros(N_trials,1) ;
  l(k).incrmt     = zeros(N_trials,1) ;
  %-- Re-initialize
  inst_mags = [params.instances(:,1) ; zeros(N_trials,1) ] ;
  inst_resps = [params.instances(:,2) ; zeros(N_trials,1) ] ;
  time_of_creation = [params.instances(:,3) ; zeros(N_trials,1) ] ;
  M = N_seed_inst ;  % current number of active entries in INST_xxxx
  curr_time = 0 ;
  %-- Main loop
  for tr = 1:N_trials
    target_mag = mags(tr,k) ;
    curr_time = curr_time + inter_trial_interval ;
    %- Calculate the base-level activations of the anchors
    T = curr_time - time_of_creation(1:M) ;  % lifetimes of the instances
    B = minus_decay .* log(T) ;      % base-level activation
    %- Add multiplicative noise to the anchor magnitudes
    noisy_inst_mags = inst_mags(1:M) .* normrnd(ones(M,1),mem_k) ;
    %- Form match scores:  Si = -|M-Ai| + H*Bi
    mismatch = abs(noisy_inst_mags - target_mag) ;
    scores = H.*B - mismatch ;
    %- Retrieve an instance according to the Boltzmann equation
    scores = scores - min(scores) ;  % make all scores have the same sign
    [win_score,curr_inst_idx] = softmax1(scores,temper) ;
    curr_inst_resp = inst_resps(curr_inst_idx) ;
    inst_mag = noisy_inst_mags(curr_inst_idx) ;
    %- Determine the correction
    delta = target_mag - inst_mag ;
    curr_incrmt = max_incrmt - sum(cutoffs > delta) ;
    %- Correct the instance response by the incrmt; incrmt can be negative
    target_resp = curr_inst_resp + curr_incrmt ;
    target_resp = max(1,min(N_cat,target_resp)) ;  % clip within bounds
    %- Register response
    responses(tr,k) = target_resp ;
    %- Store the instance, with or without feedback
    stored_resp = fdbk(tr,k) ;
    if (stored_resp==0)             % if no external feedback
      stored_resp = target_resp ;   % rehearse own response
    end
    %fprintf('trial %d, storing instance %d.\n',tr,stored_resp) ;
    M = M + 1 ;
    inst_mags(M) = target_mag ;
    inst_resps(M) = stored_resp ;
    time_of_creation(M) = curr_time ;
    %- Accumulate log
    l(k).winner(tr) = curr_inst_idx ;
    l(k).winner_mag(tr) = inst_mag ;
    l(k).winner_resp(tr) = curr_inst_resp ;
    l(k).incrmt(tr) = curr_incrmt ;
  end % of tr=1:N_trials
  l(k).memory = [inst_mags inst_resps time_of_creation] ;
end % of k=1:N_sbj

%--- Return RESPONSES and L
%%%%% End of file ANCHOR2LOG.M
